from django.core.exceptions import ValidationError, NON_FIELD_ERRORS
from django.http import Http404
from django.utils.translation import ugettext as _

from rest_framework import status
from rest_framework.response import Response
from rest_framework.settings import api_settings
from rest_framework.views import exception_handler

from .settings import HIPO_DRF_EXCEPTIONS_SETTINGS


def get_human_readable_concatenation_of(key, value):
    # "first_name", "This is required." -> "First Name: This is required."
    human_readable_key = " ".join(key.split("_")).title()
    value = f"{human_readable_key}: {value.capitalize()}"
    return value


def get_fallback_message(exception):
    if isinstance(exception, str):
        return exception
    elif isinstance(exception, list):
        for item in exception:
            if item:
                # Return first non-empty value in the list. https://github.com/Hipo/hipo-drf-exceptions/issues/8
                return get_fallback_message(item)
    elif isinstance(exception, dict):
        first_key = next(iter(exception))
        message = exception[first_key]

        # message: {"field": ["error message"]}
        if isinstance(message, list) and len(message) > 0 and isinstance(message[0], str):
            message = get_human_readable_concatenation_of(first_key, message[0])
        # message: {"field": "error message"}
        elif isinstance(message, str):
            message = get_human_readable_concatenation_of(first_key, message)

        return get_fallback_message(message)
    elif isinstance(exception, Exception):
        if hasattr(exception, "detail"):
            return get_fallback_message(exception.detail)
        elif hasattr(exception, "message"):
            # Handle Django ValidationError message attribute
            return get_fallback_message(exception.message)
        elif hasattr(exception, "message_dict"):
            return get_fallback_message(exception.message_dict)

    return exception.__str__()


def handler(exc, context):
    # It's a django validation error?
    if isinstance(exc, ValidationError):
        try:
            # Convert Django's non field errors with "__all__" key
            # to DRF's non field errors with "non_field_errors" key.
            django_non_field_errors = exc.error_dict.pop(NON_FIELD_ERRORS, None)
            if django_non_field_errors:
                exc.error_dict[HIPO_DRF_EXCEPTIONS_SETTINGS["DJANGO_NON_FIELD_ERRORS_KEY"]] = django_non_field_errors

            detail = exc.message_dict
        except AttributeError:
            detail = {
                api_settings.NON_FIELD_ERRORS_KEY: exc.messages
            }

        data = {
            'type': exc.__class__.__name__,
            'detail': detail,
            'fallback_message': get_fallback_message(exc)
        }
        response = Response(data, status=status.HTTP_400_BAD_REQUEST)
    else:  # API Exception.
        response = exception_handler(exc, context)

        if response is not None:
            if isinstance(exc, Http404):
                detail = _("Not Found.")
            else:
                detail = exc.detail

            if not isinstance(detail, dict):
                # It must be list of errors.
                if not isinstance(detail, list):
                    detail = [detail]

                detail = {
                    api_settings.NON_FIELD_ERRORS_KEY: detail
                }

            response.data = {
                'type': exc.__class__.__name__,
                'detail': detail,
                'fallback_message': get_fallback_message(exc)
            }

    return response



from django.conf import settings

from rest_framework.settings import api_settings

HIPO_DRF_EXCEPTIONS_SETTINGS = getattr(settings, "HIPO_DRF_EXCEPTIONS_SETTINGS", {})

# Defaults
HIPO_DRF_EXCEPTIONS_SETTINGS.setdefault("DJANGO_NON_FIELD_ERRORS_KEY", api_settings.NON_FIELD_ERRORS_KEY)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hipo_drf_exceptions']

package_data = \
{'': ['*']}

install_requires = \
['Django>=1.8', 'djangorestframework>=3.0']

setup_kwargs = {
    'name': 'hipo-drf-exceptions',
    'version': '0.1.6',
    'description': 'A Django app for returning consistent, verbose and easy to parse error messages on Django Rest Framework backends.',
    'long_description': '# Hipo DRF Exceptions\n[![hipo](https://img.shields.io/badge/hipo-red.svg)](https://hipolabs.com) [![Build Status](https://travis-ci.org/Hipo/hipo-drf-exceptions.svg?branch=master)](https://travis-ci.org/Hipo/hipo-drf-exceptions) [![pypi](https://img.shields.io/pypi/v/hipo-drf-exceptions.svg)](https://pypi.org/project/hipo-drf-exceptions/)\n\nA [Django](https://www.djangoproject.com) app for returning consistent, verbose and easy to parse error messages on [Django Rest Framework](https://www.django-rest-framework.org/) backends.\n\nParsing error messages generated by DRF is a bit of pain for client developers. They need to try-catch different possible error formats. When you add errors raised at the business logic level, the error parsing becomes even more difficult. \n\nThis package unifies the output format of DRF in the "Hipo Error Protocol". \n\n*No more "An error occured." errors.*\n\nThis package also provides the "fallback message", a text string that always contains a human readable error summary. This way, client developers can always fallback and show this message when the client receives an error that is not handled.\n\n> Sounds cool! Can client devs just use this field all the time?\n\nIn our past experience, we noticed that some _lazy_ client developers tend to use this message and avoid writing any code to parse the error bundle. However, the message in this field is automatically generated and may not always be suitable for end users. In order to make clear that this is a *fallback*  message, we named this field "fallback_message"\n\n\n## Table of Contents\n\n- [Installation](#installation)\n- [Usage](#usage)\n- [Client SDKs](#client-sdks)\n- [Support](#support)\n- [Contributing](#contributing)\n\n## Installation\n\nYou can get stable version of Hipo Excepitons by using pip, pipenv or poetry:\n```\npip install hipo-drf-exceptions\n```\n\n## Usage\n\n### Handler\nYou will need to set `EXCEPTION_HANDLER` of the `REST_FRAMEWORK` setting of your Django project settings.py file.\n```\nREST_FRAMEWORK = {\n    ..\n    \'EXCEPTION_HANDLER\': \'hipo_drf_exceptions.handler\',\n}\n```\n\n### Example Error Responses\n\n#### Field Error\n\nYou can make validations on model level and raise `ValidationError` when it is required.\n```python\nfrom django.core.exceptions import ValidationError\n\nclass Invitation(models.Model):\n    email = models.EmailField(unique=True)\n\n    def save(self, *args, **kwargs):\n        if User.objects.filter(email=self.email).exists():\n            raise ValidationError({"email": _("Email is already registered.")})\n            \n        super().save(*args, **kwargs)\n```\n\nIf the view or serializer encounters with the `ValidationError`, The response will be like:\n```json\n{\n    "type": "ValidationError",\n    "detail": {\n        "email": [\n            "Email is already registered."\n        ]\n    },\n    "fallback_message": "Email is already registered."\n}\n```\n\n#### Non Field Error\nImplement your own error classes.\n```python\nfrom hipo_drf_exceptions import BaseAPIException\n\nclass ProfileCredentialError(BaseAPIException):\n    default_detail = _(\'Profile credentials are not correct.\')\n```\n\nRaise error when it is required.\n```python\nclass AuthenticationView(GenericAPIView):\n\n    def post(self, request, *args, **kwargs):\n        ..\n        if not profile.check_password(password):\n            raise ProfileCredentialError()\n        ..\n```\n\nThe response will be like:\n```json\n{\n    "type": "ProfileCredentialError",\n    "detail": {\n        "non_field_errors": [\n            "Profile credentials are not correct."\n        ]\n    },\n    "fallback_message": "Profile credentials are not correct."\n}\n```\n\n## Settings\nYou can set default key for Django\'s non field errors key (it is `"__all__"`) by adding this to your Django settings:\n\n```\nHIPO_DRF_EXCEPTIONS_SETTINGS = {\n    "DJANGO_NON_FIELD_ERRORS_KEY": "field_free_errors"  # by default it\'s "non_field_errors" to be consistent with DRF.\n}\n```\n\n## Client SDKs\n\nWe have SDKs for client side.\n\n- [`hipo-exceptions-js`](https://github.com/Hipo/hipo-exceptions-js) is for JS.\n- [`hipo-exceptions-android`](https://github.com/Hipo/hipo-exceptions-android) is for Android.\n\n## Support\n\nPlease [open an issue](https://github.com/hipo/hipo-drf-exceptions/issues/new) for support.\n\n## Contributing\n\nPlease contribute using [Github Flow](https://guides.github.com/introduction/flow/). Create a branch, add commits, and [open a pull request](https://github.com/hipo/hipo-drf-exceptions/compare/).\n',
    'author': 'Hipo',
    'author_email': 'pypi@hipolabs.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hipo/hipo-drf-exceptions',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=2.7',
}


setup(**setup_kwargs)
